#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"
#include "config.h"

#include <stdio.h>
#include <errno.h>
#include <net/if.h> 
#include <netinet/in.h>
#include <sys/socket.h>

#ifdef PerlIO
typedef PerlIO * InputStream;
#else
#define PERLIO_IS_STDIO 1
typedef FILE * InputStream;
#define PerlIO_fileno(f) fileno(f)
#endif

static int
not_here(char *s)
{
    croak("%s not implemented on this architecture", s);
    return -1;
}

MODULE = IO::Socket::Multicast6	PACKAGE = IO::Socket::Multicast6

void
_mcast_add(sock,mcast_group,interface_name="")
     InputStream sock
     char* mcast_group
     char* interface_name
     PROTOTYPE: $$;$
     PREINIT:
     int fd;
     struct ipv6_mreq mreq;
     PPCODE:
     {
       int retval; 

       fd = PerlIO_fileno(sock);
       if (!inet_pton(AF_INET6, mcast_group, &mreq.ipv6mr_multiaddr))
         croak("Invalid address used for mcast group");

       mreq.ipv6mr_interface = 0; 
       if (strlen(interface_name) > 0){	 
	 mreq.ipv6mr_interface = if_nametoindex(interface_name); 
       }       
       if (setsockopt(fd, IPPROTO_IPV6,
		      IPV6_MULTICAST_IF,
		      (void*)&mreq.ipv6mr_interface,
		      sizeof(mreq.ipv6mr_interface)) < 0){
	 perror("setsockopt: ");
         croak("Set sockopt failed");	 	 
       }

       retval = setsockopt(fd, IPPROTO_IPV6,
			   IPV6_JOIN_GROUP, 
			   (void*) &mreq,
			   sizeof(mreq)); 

       if (retval < 0)
	 XSRETURN_EMPTY;
       else
	 XSRETURN_YES;
     }

void
_mcast_drop(sock,mcast_group,interface_name="")
     InputStream sock
     char* mcast_group
     char* interface_name
     PROTOTYPE: $$;$
     PREINIT:
     int fd;
     struct ipv6_mreq mreq;
     PPCODE:
     {
       int retval; 

       fd = PerlIO_fileno(sock);

       if (!inet_pton(AF_INET6, mcast_group, &mreq.ipv6mr_multiaddr.s6_addr))
         croak("Invalid address used for mcast group");

       mreq.ipv6mr_interface = 0; 
       if (strlen(interface_name) > 0){	 
	 mreq.ipv6mr_interface = if_nametoindex(interface_name); 
       }
	 
       retval = setsockopt(fd,IPPROTO_IPV6,
			   IPV6_LEAVE_GROUP, 
			   (void*)&mreq,sizeof(mreq)); 
       if (retval < 0)	 
	 XSRETURN_EMPTY;
       else
	 XSRETURN_YES;
     }

int
mcast_loopback(sock,...)
     InputStream sock
     PROTOTYPE: $;$
     PREINIT:
     int fd;
     int len;
     unsigned int previous,loopback;
     CODE:
     {
       fd = PerlIO_fileno(sock);
       /* get previous value of flag */
       len = sizeof(previous);
       if (getsockopt(fd,
		      IPPROTO_IPV6,
		      IPV6_MULTICAST_LOOP,
		      (void*)&previous,&len) < 0)
	 XSRETURN_UNDEF;
       
       if (items > 1) { /* set value */
	 loopback = SvIV(ST(1));
	 if (setsockopt(fd, IPPROTO_IPV6,
			IPV6_MULTICAST_LOOP,
			(void*)&loopback,sizeof(loopback)) < 0)
	   XSRETURN_UNDEF;
       }
       RETVAL = previous;
     }
     OUTPUT:
       RETVAL

int
mcast_ttl(sock,...)
     InputStream sock
     PROTOTYPE: $;$
     PREINIT:
     int fd;
     int len;
     unsigned int previous,ttl;
     CODE:
     {
       fd = PerlIO_fileno(sock);
       /* get previous value of flag */
       len = sizeof(previous);
       if (getsockopt(fd,IPPROTO_IPV6,
		      IPV6_MULTICAST_HOPS,
		      (void*)&previous,&len) < 0)
	 XSRETURN_UNDEF;
       
       if (items > 1) { /* set value */
	 ttl = SvIV(ST(1));
	 if (setsockopt(fd,IPPROTO_IPV6,
			IPV6_MULTICAST_HOPS,
			(void*)&ttl,sizeof(ttl)) < 0)
	   XSRETURN_UNDEF;
       }
       RETVAL = previous;
     }
     OUTPUT:
       RETVAL

void
_mcast_if(sock,...)
     InputStream sock
     PROTOTYPE: $;$
     PREINIT:
     int                fd,len;
     STRLEN             slen;
     char*              name;
     struct ipv6_mreq     mreq;
     PPCODE:
     {
       /* takes an interface name and sets of gets it. */ 

       fd = PerlIO_fileno(sock);
       if (items > 1) { /* setting interface */
	 unsigned int outif;

	 name = SvPV(ST(1),slen);
	 outif = if_nametoindex(name); 
	 if (outif < 0) 
	   croak("getsockopt() error in interface name"); 
	 
	 if (setsockopt(fd, IPPROTO_IPV6,
			IPV6_MULTICAST_IF,
			(void*)&outif,
			sizeof(outif)) == 0)
	   XSRETURN_YES;
	 else
	   XSRETURN_NO;
       } else {  /* getting interface address */

	 /* freakin' bug in Linux -- IP_MULTICAST_IF returns a struct mreqn rather than
	    an in_addr (contrary to Stevens and the setsockopt()!  
	    We work around this by looking at size of returned thing and doing a 
	    ugly cast */
	 unsigned int in_itf;
	 int retval; 
	 char buf[IFNAMSIZ]; 

	 len = sizeof(in_itf); 
	 retval = getsockopt(fd, IPPROTO_IPV6,
			     IPV6_MULTICAST_IF,
			     (void*) &in_itf, &len); 
	 if (retval < 0)
	   XSRETURN_EMPTY;

	 if (! if_indextoname(in_itf, buf)){
	   croak ("cannot convert index into name"); 
	 }
	 
	 XPUSHs(sv_2mortal(newSVpv(buf,0)));
       } 
     }
