# -*- perl -*- 
#
# DO NOT MOVE THE FIRST LINE
# It identifies the rest of the file as PERL for EMACS autoformatting
# put perl options at the end of that line, e.g., -p
#
# 
# -------------------------------------------------------------------
#                                   X-BONE
#
#                          http://www.isi.edu/xbone
#                USC Information Sciences Institute (USC/ISI)
#                   Marina del Rey, California 90292, USA
#                          Copyright (c) 1998-2005
# 
# -------------------------------------------------------------------
#
# Copyright (c) 1998-2005 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and
# its documentation in source and binary forms for non-commercial
# purposes and without fee is hereby granted, provided that the above
# copyright notice appear in all copies and that both the copyright
# notice and this permission notice appear in supporting
# documentation, and that any documentation, advertising materials,
# and other materials related to such distribution and use acknowledge
# that the software was developed by the University of Southern
# California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived
# from this software without specific prior written permission.
# 
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT
# THE SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS
# PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so
# noted when applicable.
#
# -------------------------------------------------------------------
#
# Effort partly sponsored by the Defense Advanced Research Projects
# Agency (DARPA) and Air Force Research Laboratory, Air Force Materiel
# Command, USAF, under agreement numbers F30602-98-1-0200 (X-Bone) and
# F30602-01-2-0529 (DynaBone). The views and conclusions contained
# herein are those of the authors and should not be interpreted as
# necessarily representing the official policies or endorsements,
# either expressed or implied, of the Defense Advanced Research
# Projects Agency (DARPA), the Air Force Research Laboratory, or the
# U.S. Government.
#
# This work was partly supported by the NSF STI-XTEND (ANI-0230789)
# and NETFS (ANI-0129689) projects. Any opinions, findings, and
# conclusions or recommendations expressed in this material are those
# of the authors and do not necessarily reflect the views of the
# National Science Foundation.
#
# -------------------------------------------------------------------
# $Revision: 1.3 $
#   $Author: pingali $
#     $Date: 2005/03/31 07:04:02 $
#    $State: Exp $
# ----------------------------------------------------------------------------
#
# Primary Author: Venkata Pingali 
# Description:
#  This is a wrapper to simulate NetAddr::IP 

package Net::IP::XB_IP;


require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw();

use strict;
use Math::BigInt; 
use Net::IP qw(:PROC);

use vars qw (@ISA); 
@ISA = qw( Net::IP ); 


use overload 
  eq => \&eq,
  ne => \&ne; 

# Function:
#      split_into_two
# Used by: ALL
# Description:
#	Split a block into two equal sized block
# Arguments:
#       block
# Returns:
#	two blocks of half
# Exceptions:
#	empty list 
sub split_into_two(){
  
  my $self = shift; 
  
  # just a sanity check. 
  return () if (not defined $self); 
  
  # there could be other ways of obtaining a prefix and 
  # prefixlen 
  my ($prefix, $len) = split /\//, $self->prefix(); 
  
  # you cant break a minimum block
  return () if ((($self->version() == 4) and ($len == 30))
		||(($self->version() == 6) and ($len == 126))); 
  
  $len++; 

  #first block from 0 to n/2 
  my $exp = $prefix . "/" .  $len;
  my $x = new Net::IP::XB_IP($exp); 
  
  die("cant create block for $exp") if (not defined $x); 

  # compute the first address of the second block. 
  my $intx = $x->intip + $x->size(); # get the first address as int 
  $intx = ip_inttobin($intx, $self->version()); #convert it into bin
  $intx = ip_bintoip($intx, $self->version());  # convert bin into normal
                                                # representation. 
  
  $exp = $intx . "/" .  $len;
  my $y = new Net::IP::XB_IP($exp);
  die ("undefined y") if (not defined $y); 
  
  #print "returning ", $x->prefix(), " and ", $y->prefix(), "\n";
  return ($x, $y); 
}


################################################################
#
# Exported API -- should match the split, mask, masklen similar
# to NetAddr::IP
#
################################################################


# Function:
#      split
# Used by: ALL
# Description:
#	Returns a list a Net::IP::XB_IP objects 
# Arguments:
#       Base block, blocksize
# Returns:
#	1 on success
# Exceptions:
#	"Net::IP::XB_IP::split" on error, nothing to clean up by caller
sub split($$){
  my $self = shift; 
  my $len = shift; 
  
  # you can only fix larger prefixes. 
  return () if ($len < $self->prefixlen()); 
  return $self if ($len == $self->prefixlen()); 

  my @todolist = $self->split_into_two(); 
  my @finallist = (); 

  while ($#todolist != -1){
    
    my $node = pop @todolist; 
    if ($node->prefixlen == $len){
      push @finallist, $node; 
      next; 
    }
    
    push @todolist, $node->split_into_two(); 
  } # while..
  
  return @finallist; 
}

# Function:
#      addr
# Used by: ALL
# Description:
#	returns the first valid address in the range
# Arguments:
#       
# Returns:
#	ip address
# Exceptions:
#	whatever Net::IP returns 
  
sub addr(){
  my $self = shift; 
  return $self->ip(); 
}

# Function:
#      eq
# Used by: ALL
# Description:
#	compares two XB_IP objects to see if they are equal 
# Arguments:
#       
# Returns:
#	true on success
#       false or exception 
# Exceptions:
#	
sub eq($){
  my $self = shift; 
  my $other = shift; 

  return 0 if (not defined ($other)); # 0 works as false 
  die ("unknown object type used for comparison") 
    if (ref($other) ne "Net::IP::XB_IP"); 
  
  return 
   (( $self->prefixlen() == $other->prefixlen()) and
    ( $self->addr eq $other->addr));

}

# Function:
#      ne
# Used by: ALL
# Description:
#	compares two XB_IP objects to see if they are equal 
# Arguments:
#       
# Returns:
#	false on success
#       true or exception 
# Exceptions:
#	
sub ne($){
  my $self = shift; 
  my $arg = shift; 
  return (not $self->eq($arg)); 
}


# Function:
#      split
# Used by: ALL
# Description:
#	Returns a list a Net::IP::XB_IP objects 
# Arguments:
#       Base block, blocksize
# Returns:
#	1 on success
# Exceptions:
#	"Net::IP::XB_IP::split" on error, nothing to clean up by caller
sub masklen(){
  my $self = shift; 
  return $self->prefixlen(); 
}

# Function:
#      cidr
# Used by: ALL
# Description:
#	return the prefix
# Arguments:
#       block
# Returns:
#	string
# Exceptions:
#	Net::IP::XB_IP::cidr
sub cidr(){
  my $self = shift; 
  die ("Net::IP::XB_IP::cidr") if (not defined $self); 
  return $self->prefix();
}

# Function:
#      nth
# Used by: ALL
# Description:
#	Returns a list a Net::IP::XB_IP objects 
# Arguments:
#       block, number 
# Returns:
#	an object of same type
# Exceptions:
#	Net::IP::XB_IP::nth
sub nth(){
  my $self = shift; 
  my $num = shift; 
  
  die ("Net::IP::XB_IP::nth") if (not defined $self); 

  if ((not defined $num) or ($num < 1) or ($num > $self->size() - 2)) {
    die ("Net::IP::XB_IP::nth wrong $num");
  }
  
  my $intx = $self->intip() + $num; 
  $intx = ip_inttobin($intx, $self->version()); #convert it into bin
  $intx = ip_bintoip($intx, $self->version());  # convert bin into normal
                                                # representation. 
  
  return new Net::IP::XB_IP($intx);  
}


1; 
